#ifndef _FIFOBASE_HXX
#define _FIFOBASE_HXX 1

/*
 *	fifobase.hxx
 *	ines foundation fifo class
 */

struct FifoIter {
	int _c, _i;
};
/*
 *	FifoBase
 *	Fifo management on integer basis.
 *	MultiThreading and data management has to be implemented 
 *	by the derived class.
 */
class FifoBase {
	/*
	 *	control pointers
	 */
	int	_cElCur;		/* count of elements	*/
	int	_cElMax;		/* max count of elemen  */
	int	_iData;			/* fifo (read) pointer	*/
public:
	int 	ffEmpty() { return (_cElCur == 0); }
	int 	ffFull() { return (_cElCur == _cElMax); }

	/*
	 *	get()
	 *	return index of element to be read,
	 *	and remove that element.
	 *	Fifo must be (!ffEmpty())
	 *	fWasFull is set if the Fifo was full.
	 */
	int 	iGet(int *_pfWasFull=0);

	/* 
	 *	head()/tail()
	 *  	next to get/most recently put
	 *	-- must be !empty() 
	 */
	int	iHead() { return _iData; }
	int	iTail() { return (_iData+_cElCur-1) % _cElMax; }
	
	/*
	 *	put()
	 *	return index where new element can be
	 *	stored (PUSH to TAIL).
	 *	Fifo must be (!ffFull());
	 *	fWasEmpty is set if the Fifo was empty.
	 */
	int 	iPut(int *_pfWasEmpty=0);

	/*
	 *	unput()
	 *	return index of element most recently put(),
	 *	and remove that element (POP from TAIL).
	 *	Fifo must be (!ffEmpty())
	 *	fWasFull is set if the Fifo was full.
	 */
	int 	iUnput(int *_pfWasFull);

	/*
	 *	cCur()
	 *	the count of elements currently hold in FIFO
	 */
	int cCur () { return _cElCur; }
	int cMax () { return _cElMax; }

	/*
	 *	iterate :
	 *	HT - from head(get side) to tail (put side)
	 *	TH - from tail (put side) to head(get side)
	 *	iteration must be atomic.
	 */
	void	strtIterHT(FifoIter *pit);
	int	iterHT(FifoIter*pit);	/* -1 for END		*/
	void	strtIterTH(FifoIter *pit);
	int	iterTH(FifoIter*pit);	

	FifoBase(int c); 		/* c must be at least 1	*/
	~FifoBase();			/* clear()s before deletion */
};

#endif 
