
#ifndef _GPIB_GCA_HXX_
#define _GPIB_GCA_HXX_


#include "umic/umi_driver.hxx"
#include "cplRaw/stats.hxx"
#include "gumi_msgs.hxx"
#include "regs.hxx"
#include "arch.h"
#include "gpibc/igcregs.h"
#include "gpibc/nidefs.h"
#if (defined(MMSYS_linux) || defined(MMSYS_lkern) || defined (MMSYS_mingw))
 #include <stdint.h>
#endif
#ifndef _MM_NO_IOSTREAM
  #include <iostream>
#endif

namespace INdepGpib100 {


  namespace Umi=INdepUmi100;

  /*
   * interface chip,  possibly shared by different drivers
   */
  class GifCard:public Umi::UmiCard, private Regs {

    enum {
      MAX_IREG=4,
      MAX_REG=24,
      MAX_AUX=8,
      F_ALL = 0xff,
      _END
    };

    unsigned char _mpis[MAX_IREG];		/* interrupt stats	*/

   #ifdef DEBUG
    int _mpisRef[MAX_IREG];		/* reference stats	*/
   #endif

    unsigned char _mpIst[MAX_REG];	/* interrupt status	*/
    unsigned char _mpIsh[MAX_REG];	/* interrupt shadows	*/
    unsigned char _mpor[MAX_REG];	/* output register copy	*/
    unsigned char _mpoa[MAX_AUX];	/* output aux regs	*/

    static const bool mpfIma[CNT_GCA][16];

    int _eArch;				/* controller architecture	*/
    bool _fSc;				/* system controller		*/
    bool _fIntr;			/* enable interrupt 		*/
    int _eInfo;				/* for print			*/
    Counter _ctIntr; 

    void setInfo(int e) { _eInfo=e; }

    int i_read(int i) { return _mpIst[i]=(unsigned char)(read(i)); }

    /* get the interrupt status shadow registers */
    bool isIma(int i) { return mpfIma[_eArch][i]; }
    bool isIntr() { return _fIntr; }

   public:
    bool isGood() const { return Regs::isGood(); }

    int eArch() { return _eArch; }
    bool isSc() { return _fSc; }
    bool isiGpib() const { return _eArch == GCA_72010; }
    bool isQuick() const { return _eArch == GCA_GP5130; }
    int reset(bool fSc);
    void is_clear() {  for (int i=0;i<MAX_IREG;i++) _mpis[i]=0; }

   #ifndef _MM_NO_IOSTREAM
    friend std::ostream &operator<< ( std::ostream &os, const GifCard &);
   #endif
   #ifdef DEBUG
    bool getIsChange();
   #endif


    /* non-data/non-auxiliary register access */
    void outreg( int iReg,  int iVal )  { 
      _mpor[iReg] = (unsigned char)iVal;
      if (isIntr() || !isIma(iReg) ) write(iReg, _mpor[iReg] );
    }

    /* data output/input */
    void outdat( int iVal ) { write(IGC_DIO, iVal); }
    int indat( ) { return read(IGC_DIO); }

    /* auxiliary functions accessed via auxmod */
    void outaux( int iAux, int iMask, int iVal ) {
      write( IGC_AUXMOD , iAux | (_mpoa[iAux >> 5] = (_mpoa[iAux >> 5] & ~iMask) | iVal));
    }

    int r_test(int iReg, int f ) { return read(iReg) & f; }
    int r_read(int iReg ) { return read(iReg); }

    /* loop for status bit */
    int r_loop(int iReg, int f, int fSet = 1, int c=1000);

    int isCic( ) { return  r_loop(IGC_ADSTS, IGC_ADSTS_CIC); }
    int isCac() { return r_loop(IGC_ADSTS, IGC_ADSTS_NATN, 0); }

    /* get the interface status */
    StatE getStatE( );			
    int fLinec() { return (isiGpib() || isQuick()) ? 0xff : 0 ; }
    int fLines();

    /* 
     * updating interrupt functions 
     *
     *********************************************
     *    WARNING: might reset active INT pin    *
     *********************************************
     */
    bool isIntReg(int i) {  return mpfIma[_eArch][i]; }

    //int i_test( int iReg, int f ) {
    //  return ((_mpIsh[iReg] |= i_read(iReg)) & f);
    //}

    int i_check(int i) {
      register int f;
      _mpIsh[i] |= (f=_mpIst[i]=(unsigned char)read(i));  /* get the current state, reset source */
      return (_mpor[i] & f ) ;
    }


    int i_get( int iReg, int f ) {
      int i;
      i=(_mpIsh[iReg] |= i_read(iReg));
      _mpIsh[iReg] &= ~f;
      return i & f;
    }

    int i_cur( int iReg, int f ) {
      int i;
      _mpIsh[iReg] |= (i=i_read(iReg));
      return i & f;
    }


    int i_spin( int iReg, int fTest, int fClear, int cSpin ) {
      int i;
      while (!(i=((_mpIsh[iReg] |= i_read(iReg)) & fTest)) && cSpin--);
      _mpIsh[iReg] &= ~fClear;
      return i;
    }

    bool isIrqReason();

    bool isIrq() { 
      int i;
      _mpIsh[IGC_I2] |= (i=i_read(IGC_I2));
      return (( i & 0x80 ) != 0);
    } 


    int interrupt(int iCtx);
    int timer(int iCtx, const Umi::Time &);
    char *print( char *, int ) const;


    /* 
     * non-updating interrupt functions 
     */
    int i_was( int iReg, int f ) { return _mpIsh[iReg] & f; }
    int i_last( int iReg, int f ) { return _mpIst[iReg] & f; }
    int i_clr( int iReg, int f ) {
      int i;
      i=_mpIsh[iReg];
      _mpIsh[iReg] &= ~f;
      return i & f;
    }
    int i_set( int iReg, int f ) { return _mpIsh[iReg] |= f; }

    /* commands given via auxmod */
    void outcmd( int iVal ) { write(IGC_AUXMOD, iVal); }
    void outeos( int iVal ) { write(IGC_EOS1, iVal); }
    void outspoll( int iVal ) { write(IGC_SP, iVal); }

    int cNecClk() { return 8; };	/* clock divider */
    int fIntPol() { return 0; };	/* int polarity  */

    int recvData(Umi::UmiPutBuf *pup, const GpibTermIn &gti, int cSpin);
    int sendData(Umi::UmiGetBuf *pug, const GpibTermOut &gto, int cSpin);
   
    GifCard(intptr_t addr, intptr_t eArch, int fIntr);
    ~GifCard();
  };

}
#endif

