#ifndef _WIRE_HXX
#define _WIRE_HXX 1

/*
 * \file  wire.hxx  marshalling/unmarshalling of UMI messages
 */

  #include <sys/types.h>
  #include "cplRaw/failable.hxx"


#ifdef _MM_NO_IOSTREAM
  /**
   * Host-indepedent, compact representation of UMI messages.
   */
  class Wirebuf {
    char *_pchBase;
    char *_pch;
    char *_pchLim;
   protected:
   public:
    int sbumpc() { return (_pch < _pchLim ? *((unsigned char*)(_pch++)):-1); }	
    void sputc(int ch) { if (_pch < _pchLim) *(_pch++)=(char)ch;  }
    void pubsync() {}
    size_t count() { return _pch-_pchBase; }
    Wirebuf(char *pch, size_t s):_pchBase(pch), _pch(pch), _pchLim(pch+s) {}
    ~Wirebuf() {}
  };
#else
  #include <iostream>
  typedef std::streambuf Wirebuf;
#endif

  /*
   * wire integer representation is always in network byte order
   * i.e. big-endian
   */

  /**
   * Input formatter for wire format.
   *
   * Functions to fetch octets off a Wirebuf and store it
   * in host representation.
   */
  class xistream:public Failable  {
      Wirebuf *_pwb;
      typedef unsigned u_int;
      typedef unsigned long u_long;
      Wirebuf *psb() { return _pwb; }

    #if 0
      xistream &setfailed( int fEof ) {
	 setstate(std::ios_base::failbit);
	 if (fEof) setstate(std::ios_base::eofbit);
	 return *this;
      }

      bool isBigEndWire() { return _fBE; }
      int iBigEndInt(u_char *pch, int iLen);
      int iLittleEndInt(u_char *pch, int iLen);
      unsigned getUnsigned(int iLen); 	
    #endif

      xistream &getInt(u_int &i, int iLen); 

    public:
      //xistream &getOct(u_char &);
      //xistream &peekInt1(u_int &);
      //xistream &getInt1(bool &f) { f=(getUnsigned(1)!=0); return *this; }
      //xistream &getInt1(char &ch) { ch=getUnsigned(1); return *this; }

      xistream &getInt1(u_int &i) { return getInt(i,1); }
      xistream &getInt2(u_int &i) { return getInt(i,2); }
      xistream &getInt4(u_int &i) { return getInt(i,4); }
      xistream &getInt4(long int &i) { u_int j; if (getInt(j,4)) i=j; return *this; }
      xistream &getInt4(u_long &u) { u_int j; if (getInt(j,4)) u=j; return *this; }
      xistream &getInt1(bool &f) { u_int j; if (getInt(j,1)) f=(j!=0); return *this; }
      xistream &getInt1(char &ch) { u_int j; if (getInt(j,1)) ch=(char)j; return *this; }

      //int getInt4() { u_int i; getInt(i,4); return i; }
      //xistream &getFloat4(float &r); 

      xistream (Wirebuf *);
      ~xistream ();
  };

  /**
   * Output formatter for wire format.
   *
   * Functions to put data items to a Wirebuf in a host-independent format.
   */
  class xostream:public Failable 
  {
    Wirebuf *_pwb;
    typedef unsigned u_int;
    typedef unsigned char u_char;
    
    Wirebuf *psb() { return _pwb; }
    xostream &setfailed(  ) {
      //setstate(failbit);
      return *this;
    }

    //bool isBigEndWire() { return _fBE; }
      
    //void makeBigEndInt(u_char *pch, int iLen, int i) ;
    //void makeLittleEndInt(u_char *pch, int iLen, int i) ;
    
    xostream &putInt(u_int i, int iLen);
    
  public:
    xostream &putOct(u_char );
    xostream &putInt1(u_int i) { return putInt(i,1); }
    xostream &putInt2(u_int i) { return putInt(i,2); }
    xostream &putInt4(u_int i) { return putInt(i,4); }
    xostream &putFloat4(float r);

    xostream &flush() { 
      _pwb->pubsync(); 
      return *this;
    }
    
    xostream (Wirebuf *);
    ~xostream ();
  };


#endif
