
#include "umic/umi.h"
#include "mod.h"
#include "card.h"
#include "assert.h"
#include <linux/fs.h>
#include <asm/uaccess.h>

#include <linux/version.h>
#if ( LINUX_VERSION_CODE < (KERNEL_VERSION( 2,5,0 )) )
#include <linux/sched.h>
#else
#include <linux/interrupt.h>
#endif



  /*
   * debug messages
   */
  void pcn_debug(Chan *pcn, char *szMsg, int iStat) {
    register LumiMod *pdm=pcn->_pca->_pdm;
    if (isDebug(pdm)) printk(KERN_DEBUG PFX "%s:%d %s %x (%d)\n", pdm->_szDid, pdm_getid(pdm, pcn->_pca), pcn_iChan(pcn), szMsg, iStat, iStat);
  }

  int xxx_ioctl(struct inode * pin,struct file *fp, unsigned int cmd, unsigned long arg ) {
    Chan *pcn;
    pcn=(Chan*)(fp->private_data);
    assert(pcn);
    pcn_debug( pcn, "ioctl", cmd);

    if (!access_ok(VERIFY_WRITE, arg, sizeof(UmiIoc))) return -EFAULT;

    /* check the protocol before exchanging message */
    if (cmd == GIOC_PROCHK) {
      char mpchBuf[SZ_PROTID];
      size_t s;
      UmiIoc *pio=(void*)arg;

      /* get the protocol descriptor */
      if (bufPutDown(&pio->_dd, mpchBuf, SZ_PROTID, &s)) return -EFAULT;
      if (!UmiProto_match(pcn->_pUdd->_pup, mpchBuf, strlen(mpchBuf))) return -EIO;
      return 0;
    }

    /*
     * this thread now owns pcn and pcn->_puc
     * access to the channels putdown and getup functions
     * will be single threaded.
     */
    if (cmd & GIOC_PUTMSG) {
      int iErr;

      /*
       * serialize access among concurrent threads
       * using the same channel/mode. EAGAIN is returned
       * to the thread trying to access a channel/mode
       * which is already executed.
       */
      if (pcn_lockPut(pcn)) {
	iErr=pcn_putdown(pcn, arg);
	if (iErr) {
	  pcn_debug( pcn, "putdown", iErr);
	  return pcn_unlockPut(pcn, iErr);
	}
	pcn_debug( pcn, "putdown ok", 0);
	pcn_stat(pcn, "ioctl-put-done");
	(void)pcn_unlockPut(pcn, 0);
      }
      else return -EAGAIN;
    }

    if (cmd & GIOC_GETMSG) {
	int iErr;

      if (pcn_lockGet(pcn)) {

	pcn_stat(pcn, "ioctl-get-ok");
	iErr=pcn_getup(pcn, arg);
	if (iErr) {
	  pcn_debug( pcn, "getup", iErr);
	  return pcn_unlockGet(pcn, iErr);
	}
	pcn_debug( pcn, "getup ok", 0);

	pcn_stat(pcn, "ioctl-get-done");
        (void)pcn_unlockGet(pcn,0);
      }
      else return -EAGAIN;
      pcn_stat(pcn, 0);
    }
    pcn_debug( pcn, "ioctl-done", cmd);
    return 0;
  }

  /**
   * open a UMI channel
   */
  int xxx_open(LumiMod *pdm, struct inode * pin, struct file *fp) {
    int iChanOfCard;
    int iDriverOfCard;
    int iChanOfDriver;
    int iCard;
    Card *pca;
    Chan *pcn;
    const struct UmiDriverDesc *pUdd;



    iCard=GETCARD(MINOR(pin->i_rdev));
    iChanOfCard=GETCHAN(MINOR(pin->i_rdev));
    iDriverOfCard=GETDRIV(MINOR(pin->i_rdev));
    iChanOfDriver=GETDRCN(MINOR(pin->i_rdev));

    /*
     * check if the card is available. This is concurrent
     * to the hotplug probing process.
     */

    printk(KERN_DEBUG PFX " open ca=%d, cc=%d, dr=%d, cd=%d\n", pdm->_szDid, iCard, iChanOfCard, iDriverOfCard, iChanOfDriver);
    if (iCard < 0 || iCard >= MAX_CARD) return -ENODEV;
    if (iChanOfCard < 0 || iChanOfCard >= MAX_CACN) return -ENODEV;

    pca=pdm_getcard(pdm, iCard);
    if (!pca) return -ENODEV;

    printk(KERN_DEBUG PFX " card %p\n", pdm->_szDid, pca);


    /*
     * get the driver descriptor
     */
    pUdd=UmiMod_getDriverDesc(pdm->_pum, iDriverOfCard);
    if (!pUdd) return -ENODEV;

    /*
     * init the channel
     */
    pcn=pca->_mpcn+iChanOfCard;
    pcn_init(pcn, pca,pUdd, iChanOfDriver, iChanOfCard, iDriverOfCard);
    if (pcn_open(pcn)) return -EIO;
    fp->private_data=pcn;

    pcn_debug( pcn, "open", pin->i_rdev);
    return 0;
  }

  int xxx_release( struct inode * pin, struct file *fp) {
    Chan *pcn=((Chan*)(fp->private_data));
    pcn_debug( pcn, "close", pin->i_rdev);
    pcn_close(pcn);
    return 0;
  }
